from datetime import date, timedelta
from django.db import models
from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager
from django.core.exceptions import ValidationError
from django.utils.safestring import mark_safe
from django.template.defaultfilters import date as datefilter
from django.conf import settings
from django.utils.translation import ugettext as _

from cms.models import CMSPlugin # django-cms
from cms.models.fields import PlaceholderField

from hvad.models import TranslatableModel, TranslatedFields
from hvad.manager import TranslationManager


class EventCategory(TranslatableModel):
    on_site = CurrentSiteManager(field_name='sites')
    translations = TranslatedFields(
        name = models.CharField(_(u"Name"), max_length=100)
    )
    slug = models.SlugField(_('Slug'),
            help_text=_('Name in lowercase with no spaces which will be chown in the URL of the navigator'))
    sites = models.ManyToManyField(Site)
    created = models.DateTimeField(auto_now_add=True, editable=False)
    updated = models.DateTimeField(auto_now=True, editable=False)
    
    objects = TranslationManager()
    
    def site_list(self):
        return self.sites.all()

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _(u'Event category')
        verbose_name_plural = _(u'Event categories')

class EventManager(CurrentSiteManager):	

    def upcoming(self, days=None):
        """
        Returns all Events with a start date in the next ``days`` days.
        If ``days`` is None, get all upcoming events.
        Resulting QuerySet is ordered with the soonest event first.
		"""
        now = date.today()
        if days is None:
            return self.get_query_set().filter(
                ((models.Q(end__isnull=True) | models.Q(end__gte=now)) & models.Q(start__gte=now)) |
                (models.Q(start__lte=now) & models.Q(end__gte=now))
            ).order_by('start')
        else:
            return self.get_query_set().filter(
                (models.Q(end__isnull=True) & models.Q(start__range=(now, now + timedelta(days=int(days))))) |
                (models.Q(end__isnull=False) & models.Q(start__range=(now, now + timedelta(days=int(days)))) & models.Q(end__gte=now))
            ).order_by('start')

    def past(self):
        """
        Gets all the Events that have passed.
        Queryset is ordered with the most recently past event first.
        """
        now = date.today()
        return self.get_query_set().filter(models.Q(end__lt=now) | models.Q(end__isnull=True), start__lt=now)

def format(d):
    if d.year == date.today().year:
        s = "F j"
    else:
        s = "F j, Y"
    return mark_safe(datefilter(d, s))

def isValidEndDate(field_data, all_data):
    " Validates that end date is less than (after) the start date. "
    if field_data:
        if  date(*[int(v) for v in field_data.split('-')]) <=  date(*[int(v) for v in all_data['start'].split('-')]):
            raise ValidationError(_("The end date must be after the start date.  Please select a new end date, or leave blank if this event is only one day long."))

class Event(TranslatableModel):
    on_site = EventManager(field_name='sites')
    translations = TranslatedFields(
        name = models.CharField(_("Title"), max_length=250, 
                help_text=_(u'Example: "Second Annual Charity Auction"'))
    )
    slug = models.SlugField(_('Slug'), unique_for_date='start', 
            help_text=_('Name in lowercase with no spaces which will be chown in the URL of the navigator'))
    description = PlaceholderField('event_description', blank=True, null=True, verbose_name=_('Description'),)

    start = models.DateField(_(u"Date"),
        help_text=_(u'Format: yyyy-mm-dd.  When does this event take place?  '
        'If the event is longer than one day, enter the start date here and the end date below.'))
    time = models.CharField(_("Time"), blank=True, max_length=100, 
        help_text=_(u'* Optional.  Examples: "8 am - 4 pm", "7:30 pm"'))
    # Todo -- continue to validate that the end is after the start?
    end = models.DateField(_(u'End date'), blank=True, null=True,
        help_text=_(u'* Optional.  If this event is more than one day long, enter the end date here.  Defaults to "start" date if left blank.'))
    categories = models.ManyToManyField(EventCategory, blank=True, 
        null=True, limit_choices_to={'sites__id': settings.SITE_ID})
    sites = models.ManyToManyField(Site, editable=False, 
        default=[settings.SITE_ID])
       
    created = models.DateTimeField(auto_now_add=True, editable=False)
    updated = models.DateTimeField(auto_now=True, editable=False)
    

    objects = TranslationManager()
    
    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('event_detail', None, {
            'year': self.start.strftime("%Y"),
            'month': self.start.strftime("%m"),
            'day': self.start.strftime("%d"),
            'slug': self.slug,
            })
        
    def has_passed(self):
        if self.end:
            return self.end < date.today()
        else:
            return self.start < date.today()

    def category_list(self):
        if self.categories.all():
            return ", ".join([ c.name for c in self.categories.all() ])
        return _(u'(No categories)')

    def site_list(self):
        return self.sites.all()

    def is_mutiple_days(self):
        return self.end is not None and self.end > self.start

    def get_next_upcoming(self):
        try:
            return Event.on_site.upcoming().filter(start__gte=self.start).exclude(id=self.id)[0]
        except IndexError:
            return None

    def get_previous_upcoming(self):
        try:
            return Event.on_site.upcoming().filter(start__lte=self.start).exclude(id=self.id)[0]
        except IndexError:
            return None

    def date_span(self):
        if self.end:
            s = "%s &mdash; %s" % (format(self.start), format(self.end))
        else:
            s = format(self.start)
        return mark_safe(s)
    date_span.short_description = 'date'
    date_span.admin_order_field = 'start'
    date_span.allow_tags = True
    
    class Meta:
        ordering = ('-start',)
        verbose_name = _(u'Event')
        verbose_name_plural = _(u'Events')



import mimetypes, os
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseNotModified, HttpResponseForbidden
from django.shortcuts import get_object_or_404
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.views.static import was_modified_since
from django.utils.http import http_date
from django.views.decorators.cache import never_cache
from django.db import transaction
from django.utils.encoding import smart_str
from models import *
from datetime import date


#Utils --------------------------------------------------

def render_to_response(request, template_name, context_dict = {}):
    from django.template import RequestContext
    from django.shortcuts import render_to_response as _render_to_response
    context = RequestContext(request, context_dict)
    return _render_to_response(template_name, context_instance=context)


def render_to(template_name):
    def renderer(func):
        @never_cache
        def wrapper(request, *args, **kw):
            output = func(request, *args, **kw)
            if not isinstance(output, dict):
                return output
            return render_to_response(request, template_name, output)
        return wrapper
    return renderer

def get_user_doc_or_404(request, doc_id):
    try:
        return Document.objects.get(pk=doc_id, owner=request.user)
    except:
        raise Http404()
    

#Views --------------------------------------------------

@render_to('simple_events/events_archive.html')
def events(request):
    events = Event.objects.all()
    return locals()

@render_to('simple_events/events_archive.html')
def events_day(request, year, month, day):
    events = Event.objects.filter(start=date(int(year), int(month), int(day)))
    return locals()
    
@render_to('simple_events/event_detail.html')
def events_detail(request, year, month, day, slug):
    event = Event.objects.get(start=date(int(year), int(month), int(day)), slug=slug)
    return locals()



from django.contrib import admin
from django.utils.translation import ugettext as _
from cms.admin.placeholderadmin import PlaceholderAdmin
from hvad.admin import TranslatableAdmin

from models import *

class EventCategoryAdmin(TranslatableAdmin):
    date_hierarchy = 'created'
    list_display = ('slug', '__unicode__', 'all_translations')    
    
    fieldsets = (
        ('Event category info',{
            'fields': ('name', 'slug'),
        }),
    )
    
    def __init__(self, *args, **kwargs):
        super(EventCategoryAdmin, self).__init__(*args, **kwargs)
        self.prepopulated_fields = {'slug': ('name',)}
    
    def queryset(self, request):
        return EventCategory.on_site.all()

    def save_model(self, request, obj, form, change):
        obj.save()
        from django.contrib.sites.models import Site
        current_site = Site.objects.get_current()
        if current_site not in obj.sites.all():
            obj.sites.add(current_site)

admin.site.register(EventCategory, EventCategoryAdmin)


class EventAdmin(TranslatableAdmin, PlaceholderAdmin):
    date_hierarchy = 'start'
    list_display = ('slug', '__unicode__', 'start', 'all_translations', )
    list_filter = ('start', 'categories', )
    search_fields = ['start', ]
    filter_horizontal = ['categories']
    fieldsets = (
        ('Event info',{
            'fields': ('name', 'slug', 'start', 'description'),
        }),
        (_(u'More options...'),{
            'classes': ['collapse'],
            'fields': ( 'time', 'end', 'categories'),
        }),
    )
    
    def __init__(self, *args, **kwargs):
        super(EventAdmin, self).__init__(*args, **kwargs)
        self.prepopulated_fields = {'slug': ('name',)}
    
    def queryset(self, request):
        return Event.on_site.all()
	
    def save_model(self, request, obj, form, change):
        obj.save()
        from django.contrib.sites.models import Site
        current_site = Site.objects.get_current()
        if current_site not in obj.sites.all():
            obj.sites.add(current_site)
            
admin.site.register(Event, EventAdmin)




from django.conf.urls import patterns, include, url
from models import Event

urlpatterns = patterns('',
    url(r'^$', 'simple_events.views.events', name='events'),
    url(r'^(?P<year>\d{4})/(?P<month>\d+)/(?P<day>\d+)/$',
            'simple_events.views.events_day', name="events_day"),
    url(r'^(?P<year>\d{4})/(?P<month>\d+)/(?P<day>\d+)/(?P<slug>[-\w]+)/$',
            'simple_events.views.events_detail', name="events_detail"),
)



from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _

class EventsApphook(CMSApp):
    name = _("Simple events aplication")
    urls = ["simple_events.urls"]
apphook_pool.register(EventsApphook)



from __future__ import with_statement
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from models import Event
from django.utils.translation import ugettext as _
from django.utils.safestring import mark_safe
from django.core.urlresolvers import reverse

from calendar import HTMLCalendar, TimeEncoding, month_name, day_abbr

from django.utils.dates import MONTHS
from django.utils.dates import WEEKDAYS_ABBR

from datetime import date
from itertools import groupby

from django.utils.html import conditional_escape as esc

import locale

class EventCalendar(HTMLCalendar):
    """http://journal.uggedal.com/creating-a-flexible-monthly-calendar-in-django"""
    def __init__(self, events):
        super(EventCalendar, self).__init__()
        self.events = self.group_by_day(events)

    def formatday(self, day, weekday):
        if day != 0:
            cssclass = self.cssclasses[weekday]
            if date.today() == date(self.year, self.month, day):
                cssclass += ' today'
                body = '<a class="tipsy" href="#" title="%s">%d</a>' % (_('Today'), day)
                if day in self.events:
                    d = reverse('events_day', args=[self.year, self.month, day])
                    body = '<a class="tipsy" href="%s" title="%s">%d</a>' % (d, _("Events found"), day)
                return self.day_cell(cssclass, body)

            if day in self.events:
                cssclass += ' filled'
                title = ['<ul class=event>']
                for event in self.events[day]:
                    title.append('<li><b>%s</b></li>' % esc(event.name))
                title.append('</ul>')
                d = reverse('events_day', args=(self.year, self.month, day))
                body = '<a class="tipsy" href="%s" title="%s">%d</a>' % (d, _("Events found"), day)
                return self.day_cell(cssclass, body)
            return self.day_cell(cssclass, day)
        return self.day_cell('noday', '&nbsp;')
    
    def formatweekday(self, day):
        return '<th class="%s">%s</th>' % (self.cssclasses[day], WEEKDAYS_ABBR[day].title())

    def formatmonthname(self, theyear, themonth, withyear=True):
        if withyear:
            s = '%s %s' % (MONTHS[themonth].title(), theyear)
        else:
            s = '%s' % MONTHS[themonth].title()
        return '<tr><th colspan="7" class="month">%s</th></tr>' % s

    def formatmonth(self, year, month):
        self.year, self.month = year, month
        return super(EventCalendar, self).formatmonth(year, month)

    def group_by_day(self, events):
        field = lambda event: event.start.day
        return dict(
            [(day, list(items)) for day, items in groupby(events, field)]
        )

    def day_cell(self, cssclass, body):
        return '<td class="%s">%s</td>' % (cssclass, body)


class CMSCalendarEventsPlugin(CMSPluginBase):
    
    name = _("Calendar Events")
    render_template = "simple_events/events_calendar.html"

    def render(self, context, instance, placeholder):
        today = date.today()
        year = today.year
        month = today.month

        request = context['request']
        if request.GET:
            month = int(request.GET.get('month') or month)
            year = int(request.GET.get('year') or year)
        first_day = date(year,month,1)
        if month == 12:
            last_day = date(year+1,1,1)
        else:
            last_day = date(year,month+1,1)
        events = Event.objects.filter(start__gte=first_day).filter(start__lt=last_day)
        cal = EventCalendar(events).formatmonth(year, month)
        context.update({'instance':instance,
                        'events': events,
                        'calendar': mark_safe(cal),
                        'placeholder':placeholder})
        return context

plugin_pool.register_plugin(CMSCalendarEventsPlugin)



# -*- coding: utf-8 -*-
import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.txt')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name = 'django-cms-simple-events',
    version = '0.4.1',
    packages = ['simple_events'],
    include_package_data = True,
    license = 'BSD License', # example license
    description = 'A django-cms application for managing an events calendar.',
    long_description = README,
    url = 'http://www.toporojo.es/',
    author = 'Alejandro Núñez Liz',
    author_email = 'alejandro@toporojo.es',
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    zip_safe=False,
    install_requires=['django-hvad', ],
)
